package controller

import (
	"fmt"
	"github.com/gofiber/fiber/v2"
	"goblog/dal"
	"goblog/jwt"
	"goblog/view"
	"log"
)

func Follow(c *fiber.Ctx) error {
	var req view.FollowRequest

	err := c.BodyParser(&req)
	if err != nil {
		return c.JSON(view.FollowResponse{
			BaseResponse: view.BaseResponse{
				Success: false,
				Message: "Invalid request",
			},
		})
	}

	token, err := jwt.GetJwt(c)
	_ = token
	if err != nil {
		return c.JSON(view.UserProfileResponse{
			BaseResponse: view.BaseResponse{
				Success: false,
				Message: "Not logged in",
			},
		})
	}

	if token.UserId == uint32(req.FolloweeId) {
		var verb string
		if req.Unfollow {
			verb = "unfollow"
		} else {
			verb = "follow"
		}
		return c.JSON(view.FollowResponse{
			BaseResponse: view.BaseResponse{
				Success: false,
				Message: fmt.Sprintf("You can't %v yourself", verb),
			},
		})
	}

	if req.Unfollow {
		err = dal.RemoveFollow(token.UserId, uint32(req.FolloweeId))
	} else {
		err = dal.AddFollow(token.UserId, uint32(req.FolloweeId))
	}

	if err != nil {
		log.Println("Failed to insert: ", err)
		return c.JSON(view.FollowResponse{
			BaseResponse: view.BaseResponse{
				Success: false,
				Message: "Failed to insert",
			},
		})
	}

	return c.JSON(view.FollowResponse{
		BaseResponse: view.BaseResponse{
			Success: true,
			Message: "",
		},
	})
}
