package controller

import (
	"github.com/gofiber/fiber/v2"
	"goblog/dal"
	"goblog/jwt"
	"goblog/view"
	"log"
)

func Login(c *fiber.Ctx) error {
	var req view.LoginRequest

	err := c.BodyParser(&req)
	if err != nil {
		return c.JSON(view.LoginResponse{
			BaseResponse: view.BaseResponse{
				Success: false,
				Message: "Invalid request",
			},
		})
	}

	ok, err, uid := dal.CheckUserIdentity(req.Username, req.Password)
	if err != nil {
		log.Println("Login error: ", err)
		return c.JSON(view.LoginResponse{
			BaseResponse: view.BaseResponse{
				Success: false,
				Message: "Failed to check user identity",
			},
		})
	}

	if !ok {
		return c.JSON(view.LoginResponse{
			BaseResponse: view.BaseResponse{
				Success: false,
				Message: "Invalid username or password",
			},
		})
	}

	claims := jwt.Data{
		Username: req.Username,
		UserId:   uid,
	}
	token, err := jwt.Marshal(claims)
	if err != nil {
		log.Println("Failed to marshal jwt token: ", err)
		return c.JSON(view.LoginResponse{
			BaseResponse: view.BaseResponse{
				Success: false,
				Message: "Internal error",
			},
		})
	}

	log.Println("Login success: ", claims)
	c.Cookie(&fiber.Cookie{
		Name:     jwt.CookieKey,
		Value:    token,
		HTTPOnly: false,
	})

	return c.JSON(view.LoginResponse{
		BaseResponse: view.BaseResponse{
			Success: true,
			Message: "",
		},
	})
}

func Logout(c *fiber.Ctx) error {
	c.ClearCookie(jwt.CookieKey)
	return c.JSON(view.LogoutResponse{
		BaseResponse: view.BaseResponse{
			Success: true,
			Message: "",
		},
	})
}

func IsLoggedIn(c *fiber.Ctx) error {
	token, err := jwt.GetJwt(c)
	_ = token
	return c.JSON(view.IsLoggedInResponse{
		BaseResponse: view.BaseResponse{
			Success: true,
			Message: "",
		},
		LoggedIn: err == nil,
	})
}
