package controller

import (
	"bytes"
	"fmt"
	"github.com/gofiber/fiber/v2"
	"goblog/dal"
	"goblog/jwt"
	"goblog/view"
	"log"
	"strconv"
	"text/template"
)

func isSystemUser(uid int) bool {
	return uid < 1000
}

func Profile(c *fiber.Ctx) error {
	uid, err := strconv.Atoi(c.Params("user_id"))
	if err != nil {
		return c.JSON(view.UserProfileResponse{
			BaseResponse: view.BaseResponse{
				Success: false,
				Message: "Invalid input: user id",
			},
		})
	}

	token, err := jwt.GetJwt(c)
	_ = token
	if err != nil {
		return c.JSON(view.UserProfileResponse{
			BaseResponse: view.BaseResponse{
				Success: false,
				Message: "Not logged in",
			},
		})
	}

	user, err := dal.GetUserProfile(uint32(uid))
	if err != nil {
		log.Println("Failed to get user profile: ", err)
		return c.JSON(view.UserProfileResponse{
			BaseResponse: view.BaseResponse{
				Success: false,
				Message: "Failed to get user profile",
			},
		})
	}

	followers, err := dal.GetFollowers(uint32(uid))
	if err != nil {
		log.Println("Failed to get user followers: ", err)
		return c.JSON(view.UserProfileResponse{
			BaseResponse: view.BaseResponse{
				Success: false,
				Message: "Failed to get user followers",
			},
		})
	}

	if isSystemUser(uid) {
		return c.JSON(view.UserProfileResponse{
			BaseResponse: view.BaseResponse{
				Success: false,
				Message: "Profile of system users are hidden",
			},
		})
	}

	if !user.IsProfilePublic && int(token.UserId) != uid {
		return c.JSON(view.UserProfileResponse{
			BaseResponse: view.BaseResponse{
				Success: false,
				Message: "This user's profile is private",
			},
		})
	}

	tmpl := template.New("")
	body := "Profile of user {{.Username}}:\nID: %v\nBio.: {{.Bio}}\nFollowers:\n"
	for _, follower := range followers {
		username := follower.Username
		id := follower.Id
		if id != token.UserId {
			username = "***"
		}
		body += fmt.Sprintf("%v (%v)\n", username, id)
	}
	t := fmt.Sprintf(body, uid)
	tmpl, err = tmpl.Parse(t)

	if err != nil {
		return c.JSON(view.UserProfileResponse{
			BaseResponse: view.BaseResponse{
				Success: false,
				Message: "Failed to parse template",
			},
		})
	}

	var out bytes.Buffer
	err = tmpl.Execute(&out, user)

	if err != nil {
		return c.JSON(view.UserProfileResponse{
			BaseResponse: view.BaseResponse{
				Success: false,
				Message: "Failed to render template",
			},
		})
	}

	return c.JSON(view.UserProfileResponse{
		BaseResponse: view.BaseResponse{
			Success: true,
			Message: "",
		},
		Data: out.String(),
	})
}
