package dal

import (
	"fmt"
	"goblog/model"
	"goblog/view"
)

func GetPosts(viewerUserId uint32) (posts []view.Post, err error) {
	var rawPosts []model.Post
	err = db.Select(&rawPosts, `SELECT
    	posts.id AS post_id, posts.title, posts.content,
    	posts.owner_id, posts.creation_time, posts.is_private,
    	users.id AS owner_id, users.username AS owner_username
		FROM posts
		LEFT JOIN users ON posts.owner_id = users.id
    	WHERE is_private=0 OR owner_id=$1
		`, viewerUserId)
	if err != nil {
		err = fmt.Errorf("failed to select posts: %w", err)
		return
	}

	posts = make([]view.Post, len(rawPosts))
	for i := range rawPosts {
		r := rawPosts[i]
		posts[i] = view.Post{
			Id:      r.Id,
			Title:   r.Title,
			Content: r.Content,
			Owner: view.PostOwner{
				Id:       r.OwnerId,
				Username: r.OwnerUsername,
			},
			CreationTimeMillis: r.CreationTimeMillis,
			IsPrivate:          r.IsPrivate,
		}
	}

	return
}
